//ES6 syntax

class Account {

    constructor(name, number, balance)
    {
        this.owner = name;
        this.number = number;
        this.balance = balance;
    }

    // getters
    getOwner()
    {
        return this.owner;
    }

    getNumber()
    {
        return this.number;
    }

    getBalance()
    {
        return this.balance;
    }

    getCoOwners()
    {
        // if this property is undefined, instead return the string
        return this.coOwners || "No co-owners";
    }

    printReport()
    {
        console.log(`Account #${this.number} (${this.owner}) has a balance of $${this.balance}`);
    }

    // setters
    changeOwner(newOwner)
    {
        this.owner = newOwner;
    }

    addCoOwner(name)
    {
        if(this.coOwners === undefined)
        {
            // our methods can also add new properties to our objects!
            this.coOwners = [];
        }
        this.coOwners.push(name);
        console.log(`Added ${name} as co-owner`)
    }

    deposit(amount)
    {
        console.log(`${this.getOwner()} successfully deposited $${amount}`)
        this.balance += amount;
    }

    withdraw(amount)
    {
        if (this.balance > amount)
        {
            this.balance -= amount;
            console.log(`${this.getOwner()} successfully withdrew $${amount}`)
            return true;
        }
        else
        {
            console.log(`${this.getOwner()} has insufficient funds to withdraw $${amount}`);
            return false;
        }
    }

    // making a few assumptions here about the value of "amount"
    transfer(amount, target)
    {
        if (this.withdraw(amount))
        {
            target.deposit(amount);
        }
    }
}

// set up two accounts
const john = new Account("John Smith", 10000, 75.00);
const jane = new Account("Jane Brown", 10001, 125.00);

// initial status
john.printReport();
jane.printReport();

console.log('-----')
// make some changes
john.deposit(20.00);

// invalid, then valid withdrawal
jane.withdraw(400.00);
jane.withdraw(40.00);

console.log('-----')
// next check
john.printReport();
jane.printReport();

console.log('-----')
// more changes
console.log(john.getCoOwners());
john.addCoOwner("Jack Smith");
john.addCoOwner("Judy Smith");
console.log(john.getCoOwners());

console.log('-----')
// invalid, then valid transfer
john.transfer(250.00, jane);
john.transfer(50.00, jane);

console.log('-----')
// final check
john.printReport();
jane.printReport();
